AmazoneSX4000 = {};


function AmazoneSX4000.prerequisitesPresent(specializations)
    return true;
end;

function AmazoneSX4000:load(xmlFile)

	self.setFillLevel = SpecializationUtil.callSpecializationsFunction("setFillLevel");
	self.searchPesticide = SpecializationUtil.callSpecializationsFunction("searchPesticide");
	
	self.name = Utils.getNoNil(getXMLString(xmlFile, "vehicle.name."..g_languageShort), "Amazone SX4000")
    self.typeDesc = Utils.getXMLI18N(xmlFile, "vehicle.typeDesc", "Amazone");
	
	local numWorklightsFront = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.workLightsFront#count"), 0);
    self.workLightsFront = {};
    for i=1, numWorklightsFront do
        local workLightnamei = string.format("vehicle.workLightsFront.workLightFront%d", i);
        self.workLightsFront[i] = Utils.indexToObject(self.components, getXMLString(xmlFile, workLightnamei .. "#index"));
        setVisibility(self.workLightsFront[i], false);
    end;
	
	local numWorklightsBack = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.workLightsBack#count"), 0);
    self.workLightsBack = {};
    for i=1, numWorklightsBack do
        local workLightnamei = string.format("vehicle.workLightsBack.workLightBack%d", i);
        self.workLightsBack[i] = Utils.indexToObject(self.components, getXMLString(xmlFile, workLightnamei .. "#index"));
        setVisibility(self.workLightsBack[i], false);
    end;
	
	self.particleNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.particleNode#index"));	
	self.sprayParticleSystems = {};
	
	self.sprayParticleSystem1 = {};
	local psName1 = "vehicle.sprayParticleSystem1";
	Utils.loadParticleSystem(xmlFile, self.sprayParticleSystem1, psName1, self.particleNode, false, nil, self.baseDirectory);
	table.insert(self.sprayParticleSystems, self.sprayParticleSystem1);
	
	self.sprayParticleSystem2 = {};
	local psName2 = "vehicle.sprayParticleSystem2";
	Utils.loadParticleSystem(xmlFile, self.sprayParticleSystem2, psName2, self.particleNode, false, nil, self.baseDirectory);
	table.insert(self.sprayParticleSystems, self.sprayParticleSystem2);
	
	self.sprayParticleSystem3 = {};
	local psName3 = "vehicle.sprayParticleSystem3";
	Utils.loadParticleSystem(xmlFile, self.sprayParticleSystem3, psName3, self.particleNode, false, nil, self.baseDirectory);
	table.insert(self.sprayParticleSystems, self.sprayParticleSystem3); 

	self.sprayParticleSystem4 = {};
	local psName4 = "vehicle.sprayParticleSystem4";
	Utils.loadParticleSystem(xmlFile, self.sprayParticleSystem4, psName4, self.particleNode, false, nil, self.baseDirectory);
	table.insert(self.sprayParticleSystems, self.sprayParticleSystem4);
	
	self.heightPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.heightPoint#node"));
	
	self.dungInGame = false;
	if FruitUtil.fruitIndexToDesc[FruitUtil.FRUITTYPE_PESTICIDE] ~= nil then
		self.dungInGame = true;
	end;
	
	self.fillLevel = 0;
    self.capacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.capacity"), 0.0);
	
	self.fillRootNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.fillRootNode#index"));
    if self.fillRootNode == nil then
        self.fillRootNode = self.components[1].node;
    end;
	
	self.fillTypes = {};
    self.fillTypes[FruitUtil.FRUITTYPE_UNKNOWN] = true;
	local fruitTypes = getXMLString(xmlFile, "vehicle.fillTypes#fruitTypes");
    if fruitTypes ~= nil then
        local types = Utils.splitString(" ", fruitTypes);
        for k,v in pairs(types) do
            local desc = FruitUtil.fruitTypes[v];
            if desc ~= nil then
                self.fillTypes[desc.index] = true;
            end;
        end;
    end;
	
	local hydraulicSound = getXMLString(xmlFile, "vehicle.hydraulicSound#file");
    if hydraulicSound ~= nil and hydraulicSound ~= "" then
        self.hydraulicSound = createSample("hydraulicSound");
        loadSample(self.hydraulicSound, hydraulicSound, false);
        self.hydraulicSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.hydraulicSound#pitchOffset"), 1);
        self.hydraulicSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.hydraulicSound#pitchMax"), 2.0);
        self.hydraulicSoundEnabled = false;
    end;    
	
    local spraySound = getXMLString(xmlFile, "vehicle.spraySound#file");
    if spraySound ~= nil and spraySound ~= "" then
        spraySound = Utils.getFilename(spraySound, self.baseDirectory);
        self.spraySound = createSample("spraySound");
        self.spraySoundEnabled = false;
        loadSample(self.spraySound, spraySound, false);
        self.spraySoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.spraySound#pitchOffset"), 1);
        self.spraySoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.spraySound#volume"), 1);
    end;
	
    local fillSound = getXMLString(xmlFile, "vehicle.fillSound#file");
    if fillSound ~= nil and fillSound ~= "" then
        self.fillSound = createSample("fillSound");
        loadSample(self.fillSound, fillSound, false);
        self.fillSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillSound#pitchOffset"), 1);
        self.fillSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillSound#pitchMax"), 2.0);
        self.fillSoundEnabled = false;
    end;
	
	local workSound = getXMLString(xmlFile, "vehicle.workSound#file")
	if workSound ~= nil and workSound ~= "" then
		workSound = Utils.getFilename(workSound, self.baseDirectory);
		self.workSound = createSample("workSound");
		self.workSoundEnabled = false;
		loadSample(self.workSound, workSound, false);
	end;
	
    self.currentFillType = FruitUtil.FRUITTYPE_UNKNOWN;
	self.literPerSecond = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.literPerSecond#value"), 1);
	self.massScale = 1.3*0.0001*0.7 *Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.massScale#value"), 1);
	self:setFillLevel(0, FruitUtil.FRUITTYPE_UNKNOWN);
	
	self.lightCount = 0;
	self.isOn = false;
	self.workWidth = 4;
	self.workMode = false;
	self.sprayHeight = 0.75;
	self.Speed.fold = 200;
	self.Go.fold = true;
	self.Done.fold = true;
	
end;

function AmazoneSX4000:mouseEvent(posX, posY, isDown, isUp, button)
end;

function AmazoneSX4000:keyEvent(unicode, sym, modifier, isDown)
end; 

function AmazoneSX4000:delete()
	for i=1, table.getn(self.sprayParticleSystems) do
		Utils.deleteParticleSystem(self.sprayParticleSystems[i]);
	end;
	if self.hydraulicSound ~= nil then
        delete(self.hydraulicSound);
    end;    
    if self.fillSound ~= nil then
        delete(self.fillSound);
    end;
	if self.spraySound ~= nil then
        delete(self.spraySound);
    end;
	if self.workSound ~= nil then
        delete(self.workSound);
    end;
end;

function AmazoneSX4000:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
    local fillLevel = getXMLFloat(xmlFile, key.."#fillLevel");
    local fillType = getXMLString(xmlFile, key.."#fillType");
	local mode = getXMLString(xmlFile, key.."#workMode");
	local sprayHeight = getXMLFloat(xmlFile, key.."#sprayHeight");
	local workWidth = getXMLFloat(xmlFile, key.."#workWidth");
    if fillLevel ~= nil and fillType ~= nil then
        local fillTypeDesc = FruitUtil.fruitTypes[fillType];
        if fillTypeDesc ~= nil then
			if fillType ~= "unknown" and self.dungInGame then
				self:setFillLevel(fillLevel, fillTypeDesc.index);
			else
				self:setFillLevel(self.capacity, FruitUtil.FRUITTYPE_UNKNOWN);
			end;
        end;
    end;
	if mode == "true" then
		self.workMode = not self.workMode;
		self.Speed.fold = 200;
		self.Go.fold = false;
		self.Done.fold = true;
	end;
	if sprayHeight ~= nil then
		self.sprayHeight = sprayHeight;
	end;
	if workWidth ~= nil then
		self.workWidth = workWidth;
	end;
    return BaseMission.VEHICLE_LOAD_OK;
end;

function AmazoneSX4000:getSaveAttributesAndNodes(nodeIdent)
    local fillType = "unknown";
    if self.currentFillType ~= FruitUtil.FRUITTYPE_UNKNOWN then
        fillType = FruitUtil.fruitIndexToDesc[self.currentFillType].name;
    end;
	local mode = "false";
	if self.workMode == true then
		mode = "true";
	end;
	local sprayHeight = self.sprayHeight;
	local workWidth = self.workWidth;
    local attributes = 'fillLevel="'..self.fillLevel..'" fillType="'..fillType..'" workMode="'..mode..'" sprayHeight="'..sprayHeight..'" workWidth="'..workWidth..'"';
    return attributes, nil;
end;

function AmazoneSX4000:update(dt) 
	self:anim("hub", false); 
	self:anim("fold", false); 
	self:anim("step", false); 
	for i=1, table.getn(self.componentJoints) do 	
		local joint = self.componentJoints[i];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);
	end;
	if not self.dungInGame then
		self.fillLevel = self.capacity;
	end;
	if self.fillLevel == 0 then
		self.isOn = false;
		if self.spraySoundEnabled then
			stopSample(self.spraySound);
			self.spraySoundEnabled = false;
		end;
	end;
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.AmazoneSX4000_FOLD) then
			self.Speed.fold = 1;
			self.workMode = not self.workMode;
			if not self.hydraulicSoundEnabled and self.hydraulicSound ~= nil and self:getIsActiveForSound() then
				playSample(self.hydraulicSound, 0, self.hydraulicSoundVolume, 0);
				setSamplePitch(self.hydraulicSound, self.hydraulicSoundPitchOffset-0.4);
				self.hydraulicSoundEnabled = true;   
			end;
		end;
		if InputBinding.hasEvent(InputBinding.AmazoneSX4000_STEP) then
			self.Go.step = not self.Go.step;
			self.Done.step = true;
		end;
		if InputBinding.hasEvent(InputBinding.TOGGLE_LIGHTS) then
			self.lightCount = self.lightCount+1;
			if self.lightCount > 4 then
				self.lightCount = 0;
			end;
		end;
	end;
	if self:getIsActive() then
		if self.currentPosition.fold > 53800 and not self.workMode then
			if self.hydraulicSoundEnabled then
				stopSample(self.hydraulicSound);
				self.hydraulicSoundEnabled = false;   
			end;
		elseif self.currentPosition.fold < 200 and self.workMode then
			if self.hydraulicSoundEnabled then
				stopSample(self.hydraulicSound);
				self.hydraulicSoundEnabled = false;   
			end;
		end;
		if self.lightCount == 0 then --alles aus
			for k, worklightFront in pairs(self.workLightsFront) do
				setVisibility(worklightFront, false);
			end;
			for k, worklightBack in pairs(self.workLightsBack) do
				setVisibility(worklightBack, false);
			end;
			self:setLightsVisibility(false);
		elseif self.lightCount == 1 then --fahrlicht an
			self:setLightsVisibility(true);
			for k, worklightFront in pairs(self.workLightsFront) do
				setVisibility(worklightFront, false);
			end;
			for k, worklightBack in pairs(self.workLightsBack) do
				setVisibility(worklightBack, false);
			end;
		elseif self.lightCount == 2 then -- hinten
			self:setLightsVisibility(false);
			self.lightsActive = true;
			for k, worklightFront in pairs(self.workLightsFront) do
				setVisibility(worklightFront, false);
			end;
			for k, worklightBack in pairs(self.workLightsBack) do
				setVisibility(worklightBack, true);
			end;
		elseif self.lightCount == 3 then -- vorn
			self:setLightsVisibility(false);
			self.lightsActive = true;
			for k, worklightFront in pairs(self.workLightsFront) do
				setVisibility(worklightFront, true);
			end;
			for k, worklightBack in pairs(self.workLightsBack) do
				setVisibility(worklightBack, false);
			end;
		elseif self.lightCount == 4 then --vorn+hinten
			self:setLightsVisibility(false);
			self.lightsActive = true;
			for k, worklightFront in pairs(self.workLightsFront) do
				setVisibility(worklightFront, true);
			end;
			for k, worklightBack in pairs(self.workLightsBack) do
				setVisibility(worklightBack, true);
			end;
		end;
		if self.workMode then
			self.Go.fold = false;
			self.Done.fold = true;
			if self.currentPosition.fold < 200 then
				if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
					if self.fillLevel ~= 0 then
						self.isOn = not self.isOn;
					end;
					if not self.spraySoundEnabled and self:getIsActiveForSound() then
						playSample(self.spraySound, 0, self.spraySoundVolume, 0);
						setSamplePitch(self.spraySound, self.spraySoundPitchOffset);
						self.spraySoundEnabled = true;
					end;
				end;
				if InputBinding.hasEvent(InputBinding.AmazoneSX4000_AREA) then
					if not Input.isKeyPressed(Input.KEY_shift) then
						self.workWidth = self.workWidth+1;
						if self.workWidth > 4 then
							self.workWidth = 1;
						end;
					else
						self.workWidth = self.workWidth-1;
						if self.workWidth < 1 then
							self.workWidth = 4;
						end;
					end;
				end;
				if InputBinding.hasEvent(InputBinding.AmazoneSX4000_HUB_SUB) then
					self.sprayHeight = self.sprayHeight-0.25;
				elseif InputBinding.hasEvent(InputBinding.AmazoneSX4000_HUB_ADD) then
					self.sprayHeight = self.sprayHeight+0.25;
				end;
				if self.sprayHeight > 2 then
					self.sprayHeight = 2;
				elseif self.sprayHeight < 0.75 then
					self.sprayHeight = 0.75;
				end;
				if self.isOn then
					local x,y,z = getWorldTranslation(self.cuttingAreas[self.workWidth].start);
					local x1,y1,z1 = getWorldTranslation(self.cuttingAreas[self.workWidth].width);
					local x2,y2,z2 = getWorldTranslation(self.cuttingAreas[self.workWidth].height);
					Utils.updateSprayArea(x, z, x1, z1, x2, z2);
					local difficultyMultiplier = math.max(3 * (3 - g_currentMission.missionStats.difficulty), 1);
					if self.dungInGame then
						if self.currentFillType == FruitUtil.FRUITTYPE_PESTICIDE then
							local deltaLevel = FruitUtil.fruitIndexToDesc[self.currentFillType].literPerQm * difficultyMultiplier * self.literPerSecond * self.workWidth;
							self:setFillLevel(self.fillLevel-deltaLevel, self.currentFillType);
						end;
					else
						local money = 0.05 * difficultyMultiplier * 0.02 * self.workWidth;
						g_currentMission.missionStats.money = g_currentMission.missionStats.money - money;
					end;
					if self.workWidth == 1 then
						Utils.setEmittingState(self.sprayParticleSystems[1], true);
						Utils.setEmittingState(self.sprayParticleSystems[2], false);
						Utils.setEmittingState(self.sprayParticleSystems[3], false);
						Utils.setEmittingState(self.sprayParticleSystems[4], false);
					elseif self.workWidth == 2 then
						Utils.setEmittingState(self.sprayParticleSystems[1], false);
						Utils.setEmittingState(self.sprayParticleSystems[2], true);
						Utils.setEmittingState(self.sprayParticleSystems[3], false);
						Utils.setEmittingState(self.sprayParticleSystems[4], false);
					elseif self.workWidth == 3 then
						Utils.setEmittingState(self.sprayParticleSystems[1], false);
						Utils.setEmittingState(self.sprayParticleSystems[2], false);
						Utils.setEmittingState(self.sprayParticleSystems[3], true);
						Utils.setEmittingState(self.sprayParticleSystems[4], false);
					elseif self.workWidth == 4 then
						Utils.setEmittingState(self.sprayParticleSystems[1], false);
						Utils.setEmittingState(self.sprayParticleSystems[2], false);
						Utils.setEmittingState(self.sprayParticleSystems[3], false);
						Utils.setEmittingState(self.sprayParticleSystems[4], true);
					end;
					local xH, yH, zH = getTranslation(self.heightPoint);
					local newY = (-1.15-self.sprayHeight);
					setTranslation(self.heightPoint, xH, newY, zH);
					local x,y,z = getWorldTranslation(self.heightPoint);
					local terrainHeight = 0;
					terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);	
					local differenz = y-terrainHeight; 
					if differenz > 0 then
						self.Go.hub = true;
						self.Done.hub = true;
						self.Speed.hub = 0.65;
					else
						self.Go.hub = false;
						self.Done.hub = true;
						self.Speed.hub = 0.25;
					end;
				else
					for i=1, table.getn(self.sprayParticleSystems) do
						Utils.setEmittingState(self.sprayParticleSystems[i], false);
					end;
					if self.spraySoundEnabled then
						stopSample(self.spraySound);
						self.spraySoundEnabled = false;
					end;
					self.Go.hub = false;
					self.Done.hub = true;
					self:searchPesticide();
				end;
			end;
		else
			self.isOn = false;
			for i=1, table.getn(self.sprayParticleSystems) do
				Utils.setEmittingState(self.sprayParticleSystems[i], false);
			end;
			self.Speed.hub = 1.2;
			self.Go.hub = false;
			self.Done.hub = true;
			if self.currentPosition.hub < 200 then
				self.Go.fold = true;
				self.Done.fold = true;
				self:searchPesticide();
			end;
			if self.spraySoundEnabled then
				stopSample(self.spraySound);
				self.spraySoundEnabled = false;
			end;
		end;
	end;
end;

function AmazoneSX4000:draw()
	local trailerFillLevel, trailerCapacity = self:getAttachedTrailersFillLevelAndCapacity();
    if trailerFillLevel ~= nil and trailerCapacity ~= nil then
        self:drawGrainLevel(trailerFillLevel+self.fillLevel, trailerCapacity+self.capacity, 101);
	else
		self:drawGrainLevel(self.fillLevel, self.capacity, 101);
    end;
	if self.workMode and self.currentPosition.fold < 200 then
		g_currentMission:addExtraPrintText("Klavesa "..string.format("%s: Postrikovy zaber  %d m aktivovan", InputBinding.getButtonKeyName(InputBinding.AmazoneSX4000_AREA), self.workWidth*9).."");
		g_currentMission:addExtraPrintText("Klavesa "..string.format("%s/%s: Vyska striku: %.2f m nad zemi", InputBinding.getButtonKeyName(InputBinding.AmazoneSX4000_HUB_ADD), InputBinding.getButtonKeyName(InputBinding.AmazoneSX4000_HUB_SUB),  self.sprayHeight).."");
		if self.isOn then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("AmazoneSX40001"), InputBinding.AmazoneSX4000_FOLD);
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
		end;
	else
		g_currentMission:addHelpButtonText(g_i18n:getText("AmazoneSX40002"), InputBinding.AmazoneSX4000_FOLD);
	end;
	if self.CheckDone.step then
		g_currentMission:addHelpButtonText(g_i18n:getText("AmazoneSX40004"), InputBinding.AmazoneSX4000_STEP);
	else
		g_currentMission:addHelpButtonText(g_i18n:getText("AmazoneSX40003"), InputBinding.AmazoneSX4000_STEP);
	end;
	if self.currentFillType ~= FruitUtil.FRUITTYPE_UNKNOWN then
        local overlay = g_currentMission.fruitOverlays[self.currentFillType];
        if overlay ~= nil then
            g_currentMission.fruitOverlays[self.currentFillType]:render();
        end;
    end;
end;

function AmazoneSX4000:setFillLevel(fillLevel, fillType)
    self.currentFillType = fillType;
    self.fillLevel = fillLevel;
    if self.fillLevel > self.capacity then
        self.fillLevel = self.capacity;
    end;
    if self.fillLevel < 0 then
        self.fillLevel = 0;
        self.currentFillType = FruitUtil.FRUITTYPE_UNKNOWN;
    end;
end;

function AmazoneSX4000:onDeactivate()
    AmazoneSX4000.onDeactivateSounds(self);
	self.isOn = false;
end;

function AmazoneSX4000:onDeactivateSounds()
    if self.fillSoundEnabled and self.fillLevel == 0 then
        stopSample(self.fillSound);
        self.fillSoundEnabled = false;   
    end;
    if self.hydraulicSoundEnabled then
        stopSample(self.hydraulicSound);
        self.hydraulicSoundEnabled = false;   
    end;
	if self.spraySoundEnabled then
	    stopSample(self.spraySound);
	    self.spraySoundEnabled = false;
	end;
	if self.workSoundEnabled then
	    stopSample(self.workSound);
	    self.workSoundEnabled = false;
	end;
end;

function AmazoneSX4000:onLeave()
	for k, worklightFront in pairs(self.workLightsFront) do
		setVisibility(worklightFront, false);
	end;
	for k, worklightBack in pairs(self.workLightsBack) do
		setVisibility(worklightBack, false);
	end;
	self.lightCount = 0;
	AmazoneSX4000.onDeactivate(self);
	for i=1, table.getn(self.sprayParticleSystems) do
		Utils.setEmittingState(self.sprayParticleSystems[i], false);
	end;
end;

function AmazoneSX4000:searchPesticide()
	self.inRange = false;
	local nearestDistance = 6.7;
	local px, py, pz = getWorldTranslation(self.fillRootNode);
	for i=1, table.getn(g_currentMission.vehicles) do
		local vx, vy, vz = getWorldTranslation(g_currentMission.vehicles[i].rootNode);
		local distance = Utils.vector3Length(px - vx, py - vy, pz - vz);
		if distance < nearestDistance then
			for fruitTypes, v in pairs(self.fillTypes) do
				local attachable = g_currentMission.vehicles[i];
				if fruitTypes == attachable.currentFillType and attachable.rootNode ~= self.rootNode and self.fillLevel ~= self.capacity and attachable.fillLevel > 0 then
					self.inRange = true;
					if not self.workSoundEnabled then
					  playSample(self.workSound, 0, 1, 0);
					  self.workSoundEnabled = true;
					end;
					local difficultyMultiplier = math.max(3 * (3 - g_currentMission.missionStats.difficulty), 1);
					local money = FruitUtil.fruitIndexToDesc[attachable.currentFillType].pricePerLiter * difficultyMultiplier * attachable.literPerSecond;
					g_currentMission.missionStats.money = g_currentMission.missionStats.money - money;
					local newLevel = self.fillLevel + attachable.literPerSecond;
					self:setFillLevel(newLevel, attachable.currentFillType);
				end;
			end;
		end;
	end;
	if not self.inRange and self.workSoundEnabled then
		stopSample(self.workSound);
		self.workSoundEnabled = false;
	end;
end;